if (!lemonade && typeof (require) === 'function') {
    var lemonade = require('lemonadejs');
}

;(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    global.lazyLoading = factory();
}(this, (function () {

    'use strict';

    /**
     * Compare two arrays to see if contains exact the same elements
     * @param {number[]} a1
     * @param {number[]} a2
     */
    const compareArray = function (a1, a2) {
        if (! a1 || ! a2) {
            return false;
        }

        let i = a1.length;
        if (i !== a2.length) {
            return false;
        }
        while (i--) {
            if (a1[i] !== a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * Manage items in the viewport based on the scrolling
     * @returns {string}
     * @constructor
     */
    const Lazyloading = function (template) {
        let self = this;

        // Default row height
        let defaultRowHeight = 28;

        // Visible rows
        self.visibleRows = [];

        /**
         * Get the position from top of a row by its index
         * @param item
         * @returns {number}
         */
        const getRowPosition = function(item) {
            // Position from top
            let top = 0;
            if (item) {
                let items = self.data;
                if (items && items.length) {
                    let index = self.data.indexOf(item);
                    // Go through the items
                    for (let j = 0; j < index; j++) {
                        top += items[j].height || defaultRowHeight;
                    }
                }
            }
            return top;
        }

        const updateScroll = function() {
            let items = self.data;
            if (items && items.length) {
                // Total of items in the container
                let numOfItems = items.length;
                // Position from top
                let height = 0;
                // Go through the items
                for (let j = 0; j < numOfItems; j++) {
                    height += items[j].height || defaultRowHeight;
                }
                // Update height
                self.scroll.style.height = height + 'px';
            }
        }

        const getVisibleRows = function(reset) {
            let items = self.data;
            if (items) {
                let rows = [];
                // Total of items in the container
                let numOfItems = items.length;
                // Height
                let height = 0;
                // Controls
                let y = self.el.scrollTop;
                let h = y + (self.height || self.container.offsetHeight);
                // Go through the items
                for (let j = 0; j < numOfItems; j++) {
                    if (items[j].visible !== false) {
                        // Height
                        let rowHeight = items[j].height || defaultRowHeight;
                        // Return on partial width
                        if (height + rowHeight > y && height < h) {
                            rows.push(items[j]);
                        }
                        height += rowHeight;
                    }
                }

                // Update visible rows
                if (reset || ! compareArray(rows, self.visibleRows)) {
                    self.visibleRows = rows;
                    // Adjust scroll height
                    let adjustScroll = reset;
                    // Adjust scrolling
                    for (let i = 0; i < rows.length; i++) {
                        // Item
                        let item = rows[i];
                        // Item height
                        let h = item.el.offsetHeight;
                        // Update row height
                        if (! item.height || h !== item.height) {
                            // Keep item height
                            item.height = h
                            // Adjust total height
                            adjustScroll = true;
                        }
                    }

                    // Update scroll if the height of one element has been changed
                    if (adjustScroll) {
                        updateScroll();
                    }
                }

                // Adjust position of the first element
                let position = getRowPosition(self.visibleRows[0]);
                let diff = position - self.el.scrollTop;
                if (diff > 0) {
                    diff = 0;
                }
                self.container.style.top = diff + 'px';
            }
        }

        /**
         * Move the position to the top and re-render based on the scroll
         * @param reset
         */
        const render = function (reset) {
            // Move scroll to the top
            self.el.scrollTop = 0;
            // Append first batch
            getVisibleRows(reset);
            // Call event
            if (typeof(self.onupdate) === 'function') {
                self.onupdate.call(self, self.el);
            }
        }

        self.adjustPosition = function(item) {
            let y = self.el.scrollTop;
            let h = y + (self.height || self.container.offsetHeight);

            console.log(y,item.offsetTop)
        }

        // Controls
        self.scrollControls = function(e) {
            // Get all items that should be visible based on the position of the scroll
            getVisibleRows(false);
        }

        self.setData = function (items) {
            if (items) {
                self.data = items;
            }
        }

        self.goto = function(item) {
            self.el.scrollTop = getRowPosition(item);
        }

        self.onchange = function(prop) {
            if (prop === 'data') {
                render(true);
            }
        }

        return `<div class="lm-lazy" :data="self.data" onscroll="self.scrollControls" onwheel="self.scrollControls">
            <div class="lm-lazy-scroll" :ref="self.scroll"></div>
            <div class="lm-lazy-items" :ref="self.container" :loop="self.visibleRows">${template}</div>
        </div>`;
    }

    lemonade.setComponents({ Lazyloading: Lazyloading });

    return function (root, options) {
        if (typeof (root) === 'object') {
            let template = root.innerHTML;
            root.innerHTML = '';
            lemonade.render(Lazyloading, root, options, template)
            return options;
        } else {
            return Lazyloading;
        }
    }

})));